@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2011 Dachary <loic@dachary.org>
+# Copyright (C) 2011 Loic Dachary <loic@dachary.org>
 #
 # This software's license gives you freedom; you can copy, convey,
 # propagate, redistribute and/or modify this program under the terms of
@@ -21,6 +21,7 @@ import os
 sys.path.insert(0, os.path.abspath("..")) # so that for M-x pdb works
 import sqlite3
 
+from twisted.python import log
 from twisted.trial import unittest, runner, reporter
 from twisted.internet import defer, reactor
 from twisted.web import server, resource, http
@@ -30,15 +31,65 @@ from cardstories.service import CardstoriesService
 from twisted.internet import base
 base.DelayedCall.debug = True
 
+class CardstoriesServiceTestNotify(unittest.TestCase):
+    
+    def test00_notify(self):
+        service = CardstoriesService({})
+        d = service.listen()
+        def check(result):
+            self.assertTrue(result)
+            service.checked = True
+            return result
+        d.addCallback(check)
+        service.notify(True)
+        self.assertTrue(service.checked)
+
+    def test01_notify_recursive(self):
+        service = CardstoriesService({})
+        d = service.listen()
+        def recurse(result):
+            try:
+                service.notify(False)
+            except UserWarning, e:
+                self.failUnlessSubstring('recurs', e.args[0])
+                service.recursed = True
+            return result
+        d.addCallback(recurse)
+        service.notify(True)
+        self.assertTrue(service.recursed)
+
+    def test02_notify_ignore_exception(self):
+        service = CardstoriesService({})
+        d = service.listen()
+        def fail(result):
+            service.raised = True
+            raise UserWarning, 'raise exception'
+        d.addCallback(fail)
+        service.notify(True)
+        self.assertTrue(service.raised)
+
 class CardstoriesServiceTestInit(unittest.TestCase):
 
     def test00_startService(self):
         database = 'test.sqlite'
         service = CardstoriesService({'db': database})
         self.assertFalse(os.path.exists(database))
+        def start(event):
+            self.assertEqual(event['type'], 'start')
+            service.notified_start = True
+            return event
+        service.listen().addCallback(start)
         service.startService()
+        self.assertTrue(service.notified_start)
         self.assertTrue(os.path.exists(database))
-        return service.stopService()
+        def stop(event):
+            self.assertEqual(event['type'], 'stop')
+            service.notified_stop = True
+            return event
+        service.listen().addCallback(stop)
+        d = service.stopService()
+        self.assertTrue(service.notified_stop)
+        return d
 
     @defer.inlineCallbacks
     def test01_load(self):
@@ -65,6 +116,7 @@ class CardstoriesServiceTestInit(unittest.TestCase):
         service.startService()
         self.assertEquals(game_id, service.games[game_id].id)
         self.assertEquals([player_id], service.games[game_id].get_players())
+        self.assertEquals(len(service.games[game_id].pollers), 1)
         yield service.stopService()
 
 class CardstoriesServiceTest(unittest.TestCase):
@@ -82,22 +134,6 @@ class CardstoriesServiceTest(unittest.TestCase):
         os.unlink(self.database)
         return self.service.stopService()
 
-class CardstoriesServiceTestRun(unittest.TestCase):
-
-    def setUp(self):
-        self.database = 'test.sqlite'
-        if os.path.exists(self.database):
-            os.unlink(self.database)
-
-    def tearDown(self):
-        os.unlink(self.database)
-
-    def test00_run(self):
-        self.service = CardstoriesService({'db': self.database, 'loop': 2, 'click': 0.1})
-        d = self.service.startService()
-        d.addCallback(lambda result: self.assertTrue(result))
-        return d
-
 class CardstoriesServiceTestHandle(CardstoriesServiceTest):
 
     def test01_required(self):
@@ -109,7 +145,7 @@ class CardstoriesServiceTestHandle(CardstoriesServiceTest):
     @defer.inlineCallbacks
     def test02_handle(self):
         for action in self.service.ACTIONS:
-            result = yield self.service.handle({ 'action': [action] })
+            result = yield self.service.handle(None, { 'action': [action] })
             self.failUnlessSubstring(action, result['error'])
             self.failUnlessSubstring('must be given', result['error'])
 
@@ -348,6 +384,7 @@ class CardstoriesServiceTest(CardstoriesServiceTest):
                 'modified': self.service.games[game2].modified
                 })
 
+    @defer.inlineCallbacks
     def test06_get_or_create_player(self):
         # create a player that did not exist
         self.assertEquals({}, self.service.players)
@@ -376,11 +413,13 @@ class CardstoriesServiceTest(CardstoriesServiceTest):
         player.access_time = 0 # pretend the player has not been accessed for a long time
         func = player.timer.func
         player.timer.cancel()
-        self.assertTrue(func())
+        result = yield func()
+        self.assertTrue(result)
         self.assertTrue(player.deleted)
         self.assertFalse(self.service.players.has_key(player_id))
         # timeout on a deleted player does nothing
-        self.assertFalse(func())
+        result = yield func()
+        self.assertFalse(result)
 
     @defer.inlineCallbacks
     def test07_game_notify(self):
@@ -408,13 +447,32 @@ class CardstoriesServiceTest(CardstoriesServiceTest):
             self.assertEquals(result['modified'], [player.modified])
             self.assertEquals(result['player_id'], [owner_id])
             self.assertEquals(result['game_id'], [game.id])
+            game.checked = True
             return result
         d.addCallback(check)
-        game.touch() # calls game_notify indirectly
+        def change(event):
+            self.assertTrue(event['type'], 'change')
+            self.assertTrue(event['game'].get_id(), game.get_id())
+            game.changed = True
+        self.service.listen().addCallback(change)
+        yield game.touch() # calls game_notify indirectly
+        self.assertTrue(game.checked)
+        self.assertTrue(game.changed)
+        #
+        # Event notification when a game is destroyed
+        #
+        def destroy(event):
+            self.assertTrue(event['type'], 'delete')
+            self.assertTrue(event['game'].get_id(), game.get_id())
+            game.destroyed = True
+        self.service.listen().addCallback(destroy)            
+        game.destroy()
+        self.assertTrue(game.destroyed)
         #
         # calling game_notify on a non existent game is a noop
         #
-        self.assertFalse(self.service.game_notify({}, 200))
+        result = yield self.service.game_notify({}, 200)
+        self.assertFalse(result)
 
     @defer.inlineCallbacks
     def test08_poll(self):
@@ -439,7 +497,7 @@ class CardstoriesServiceTest(CardstoriesServiceTest):
             player.ok = True
             return result
         d.addCallback(check)
-        player.touch({'ok': True})
+        yield player.touch({'ok': True})
         self.assertTrue(player.ok)
         #
         # poll game
@@ -459,7 +517,7 @@ class CardstoriesServiceTest(CardstoriesServiceTest):
             game.ok = True
             return result
         d.addCallback(check)
-        game.touch()
+        yield game.touch()
         self.assertTrue(game.ok)
         
 
@@ -487,15 +545,100 @@ class CardstoriesServiceTest(CardstoriesServiceTest):
         yield game.cancel()
         self.assertFalse(self.service.games.has_key(game.id))
 
+    @defer.inlineCallbacks
+    def test10_invite(self):
+        card = 5
+        sentence = 'SENTENCE'
+        owner_id = 15
+        game = yield self.service.create({ 'action': ['create'],
+                                           'card': [card],
+                                           'sentence': [sentence],
+                                           'owner_id': [owner_id]})
+        players = [ 16, 17 ]
+        for player_id in players:
+            yield self.service.participate({ 'action': ['participate'],
+                                             'player_id': [player_id],
+                                             'game_id': [game['game_id']] })
+
+        gameId = game['game_id']
+        yield self.service.invite({ 'action': ['invite'],
+                                    'game_id': [gameId],
+                                    'owner_id': [owner_id] })
+        game = self.service.games[game['game_id']]
+        self.assertEquals([owner_id] + players, game.get_players())
+
+        invited = 20
+        yield self.service.invite({ 'action': ['invite'],
+                                    'game_id': [gameId],
+                                    'player_id': [invited],
+                                    'owner_id': [owner_id] })
+        game = self.service.games[gameId]
+        self.assertEquals([owner_id] + players + [invited], game.get_players())
+
+        yield game.cancel()
+        self.assertFalse(self.service.games.has_key(game.id))
+
+    @defer.inlineCallbacks
+    def test11_complete_and_poll(self):
+        winner_card = 5
+        sentence = 'SENTENCE'
+        owner_id = 15
+        result = yield self.service.create({ 'card': [winner_card],
+                                           'sentence': [sentence],
+                                           'owner_id': [owner_id]})
+        for player_id in ( 16, 17 ):
+            yield self.service.participate({ 'action': ['participate'],
+                                             'player_id': [player_id],
+                                             'game_id': [result['game_id']] })
+            player = yield self.service.player2game({ 'action': ['player2game'],
+                                                      'player_id': [player_id],
+                                                      'game_id': [result['game_id']] })
+            card = player['cards'][0]
+            yield self.service.pick({ 'action': ['pick'],
+                                      'player_id': [player_id],
+                                      'game_id': [result['game_id']],
+                                      'card': [card] })
+        
+        yield self.service.voting({ 'action': ['voting'],
+                                    'game_id': [result['game_id']],
+                                    'owner_id': [owner_id] })
+
+        c = self.db.cursor()
+        c.execute("SELECT board FROM games WHERE id = %d" % result['game_id'])
+        board = c.fetchone()[0]
+        winner_id = 16
+        yield self.service.vote({ 'action': ['vote'],
+                                  'game_id': [result['game_id']],
+                                  'player_id': [winner_id],
+                                  'card': [winner_card] })
+        loser_id = 17
+        yield self.service.vote({ 'action': ['vote'],
+                                  'game_id': [result['game_id']],
+                                  'player_id': [loser_id],
+                                  'card': [120] })
+        self.assertTrue(self.service.games.has_key(result['game_id']))
+        game = self.service.games[result['game_id']]
+        d = self.service.poll({'action': ['poll'],
+                               'modified': [game.modified],
+                               'game_id': [game.id]})
+        def check(result):
+            self.assertEquals([game.id], result['game_id'])
+            return result
+        d.addCallback(check)
+        yield self.service.complete({ 'action': ['complete'],
+                                      'game_id': [result['game_id']],
+                                      'owner_id': [owner_id] })
+        self.assertFalse(self.service.games.has_key(result['game_id']))
+        yield d
+
 def Run():
     loader = runner.TestLoader()
-#    loader.methodPrefix = "test09_"
+#    loader.methodPrefix = "test11_"
     suite = loader.suiteFactory()
+    suite.addTest(loader.loadClass(CardstoriesServiceTestNotify))
     suite.addTest(loader.loadClass(CardstoriesServiceTestInit))
     suite.addTest(loader.loadClass(CardstoriesServiceTest))
     suite.addTest(loader.loadClass(CardstoriesServiceTestHandle))
-    suite.addTest(loader.loadClass(CardstoriesServiceTestRun))
-
     return runner.TrialRunner(
         reporter.VerboseTextReporter,
         tracebackFormat='default',
